"use strict";

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Chart = exports.fixDatasetStyle = exports.initChartDataFromData = exports.pointIntoChart = exports.adjustDatasetsToPayload = exports.getDefaultDataset = exports.commonOptions = exports.chartColors = void 0;

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _chart = _interopRequireDefault(require("chart.js"));

var R = _interopRequireWildcard(require("ramda"));

var _subSeconds = _interopRequireDefault(require("date-fns/fp/subSeconds"));

var _transforms = require("../transforms");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

// import { generateString } from '../data-utils';
var defPointRadius = 1.3;
var chartColors = ['rgb(50, 152, 235)', 'rgb(255, 159, 64)', 'rgb(75, 185, 92)', 'rgb(153, 102, 255)', 'rgb(255, 99, 99)', 'rgb(255, 205, 86)', 'rgb(190, 90, 60)', 'rgb(231, 80, 200)', 'rgb(60, 203, 180)', 'rgb(255, 20, 0)', 'rgb(201, 203, 207)'];
exports.chartColors = chartColors;
var defaultCanvasStyle = {
  width: 300,
  height: 300
};
var commonOptions = {
  animation: {
    duration: 200
  },
  hover: {
    animationDuration: 0 // duration of animations when hovering an item

  },
  legend: {
    labels: {
      boxWidth: 6
    }
  },
  maintainAspectRatio: false,
  responsive: true,
  responsiveAnimationDuration: 0 // animation duration after a resize

};
exports.commonOptions = commonOptions;

var getDefaultDataset = function getDefaultDataset(idx) {
  var data = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

  var _ref = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {},
      _ref$colors = _ref.colors,
      colors = _ref$colors === void 0 ? chartColors : _ref$colors,
      borderWidth = _ref.borderWidth,
      _ref$pointRadius = _ref.pointRadius,
      pointRadius = _ref$pointRadius === void 0 ? defPointRadius : _ref$pointRadius,
      _ref$fill = _ref.fill,
      fill = _ref$fill === void 0 ? false : _ref$fill;

  return {
    fill: fill,
    label: "pl[".concat(idx, "]"),
    data: data,
    backgroundColor: colors[idx],
    borderColor: colors[idx],
    borderWidth: borderWidth,
    pointRadius: pointRadius,
    spanGaps: true
  };
};

exports.getDefaultDataset = getDefaultDataset;

var adjustDatasetsToPayload = function adjustDatasetsToPayload(datasets, payload, datasetOptions) {
  /* eslint-disable fp/no-loops, fp/no-mutating-methods */
  while (datasets.length < payload.length) {
    datasets.push(getDefaultDataset(datasets.length, R.repeat(null, datasets[0] ? datasets[0].data.length : 0), datasetOptions));
  }

  while (datasets.length > payload.length) {
    datasets.pop();
  }
  /* eslint-enable fp/no-loops, fp/no-mutating-methods */

};

exports.adjustDatasetsToPayload = adjustDatasetsToPayload;

var pointToX = function pointToX(point) {
  var _ref2, _ref3, _ref4, _point$x;

  return (_ref2 = (_ref3 = (_ref4 = (_point$x = point === null || point === void 0 ? void 0 : point.x) !== null && _point$x !== void 0 ? _point$x : point === null || point === void 0 ? void 0 : point.ts) !== null && _ref4 !== void 0 ? _ref4 : point === null || point === void 0 ? void 0 : point.timestamp) !== null && _ref3 !== void 0 ? _ref3 : point === null || point === void 0 ? void 0 : point.time) !== null && _ref2 !== void 0 ? _ref2 : new Date();
};

var pointToYList = function pointToYList(point) {
  var _ref5, _ref6, _point$payload;

  return (_ref5 = (_ref6 = (_point$payload = point === null || point === void 0 ? void 0 : point.payload) !== null && _point$payload !== void 0 ? _point$payload : point === null || point === void 0 ? void 0 : point.pl) !== null && _ref6 !== void 0 ? _ref6 : point === null || point === void 0 ? void 0 : point.data) !== null && _ref5 !== void 0 ? _ref5 : [null];
};

var pointIntoChart = R.curry(function (chart, _ref7, point) {
  var length = _ref7.length,
      ttl = _ref7.ttl,
      datasetOptions = _ref7.datasetOptions;
  var x = pointToX(point);
  var ys = pointToYList(point); // Add or remove datasets depending on number of ys:

  adjustDatasetsToPayload(chart.data.datasets, ys, datasetOptions); // Calculate if we should cut the tail

  var chronologicStart = (0, _transforms.nilOrEmpty)(chart.data.labels) || x > chart.data.labels[0];
  var remove = ttl ? R.findIndex(R.lt((0, _subSeconds["default"])(ttl)(x)))(chart.data.labels) : length && (!chronologicStart || chart.data.labels.length >= length) ? 1 : 0;
  /* eslint-disable fp/no-mutating-methods */

  chart.data.labels.splice(0, remove);
  chart.data.labels.push(x);
  chart.data.datasets.forEach(function (ds, ix) {
    var val = ys[ix];
    ds.data.splice(0, remove);
    ds.data.push(val);
  });
  /* eslint-enable fp/no-mutating-methods */
});
exports.pointIntoChart = pointIntoChart;
var propTypes = {
  data: _propTypes["default"].objectOf(_propTypes["default"].any),
  style: _propTypes["default"].objectOf(_propTypes["default"].any),
  type: _propTypes["default"].string,
  options: _propTypes["default"].objectOf(_propTypes["default"].any),
  point: _propTypes["default"].objectOf(_propTypes["default"].any),
  points: _propTypes["default"].arrayOf(_propTypes["default"].object),
  length: _propTypes["default"].number,
  ttl: _propTypes["default"].number
};

var getDefaultOptions = function getDefaultOptions() {
  return _objectSpread({}, commonOptions, {
    // tooltips: { enabled: false },
    scales: {
      xAxes: [{
        type: 'time',
        time: {
          parser: 'HH:mm:ss',
          tooltipFormat: 'D MMM HH:mm:ss',
          minUnit: 'second',
          displayFormats: {
            millisecond: 'mm:ss.SSS',
            second: 'HH:mm:ss',
            minute: 'HH:mm',
            hour: 'D HH:00'
          }
        },
        scaleLabel: {
          display: false // labelString: 'Date',

        }
      }],
      yAxes: [{
        scaleLabel: {
          display: false // labelString: 'value',

        }
      }]
    }
  });
};

var initChartDataFromData = function initChartDataFromData() {
  var datasetInit = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
  return R.compose(R.over(R.lensProp('datasets'), R.when(R.either(R.complement(Array.isArray), R.isEmpty), function () {
    return datasetInit;
  })), R.when(_transforms.nilOrEmpty, function () {
    return {};
  }), R.clone);
};

exports.initChartDataFromData = initChartDataFromData;

var fixDatasetStyle = function fixDatasetStyle(datasetOptions) {
  return function (ds, ix) {
    /* eslint-disable no-param-reassign, fp/no-mutation */
    R.mapObjIndexed(function (val, key) {
      ds[key] = ds[key] || val;
    })(getDefaultDataset(ix, ds.data, datasetOptions));
    /* eslint-enable no-param-reassign, fp/no-mutation */
  };
};

exports.fixDatasetStyle = fixDatasetStyle;
var defaultData = {};
var defaultOptions = {};
var defaultStyle = {};

var Chart = function Chart(_ref8) {
  var _ref8$style = _ref8.style,
      style = _ref8$style === void 0 ? defaultStyle : _ref8$style,
      _ref8$type = _ref8.type,
      type = _ref8$type === void 0 ? 'line' : _ref8$type,
      _ref8$data = _ref8.data,
      data = _ref8$data === void 0 ? defaultData : _ref8$data,
      _ref8$options = _ref8.options,
      options = _ref8$options === void 0 ? defaultOptions : _ref8$options,
      point = _ref8.point,
      points = _ref8.points,
      length = _ref8.length,
      ttl = _ref8.ttl;
  var canvasRef = (0, _react.useRef)(null);
  var chart = (0, _react.useRef)(null); // const chartId = useRef(generateString(8));

  (0, _react.useEffect)(function () {
    // console.log('Chart init:', type, data, options, point, points, length, ttl);
    if (chart.current) return;
    var chartData = initChartDataFromData()(data);
    var fixDS = fixDatasetStyle();
    chartData.datasets.forEach(fixDS);
    chart.current = new _chart["default"](canvasRef.current, {
      type: type,
      data: chartData,
      options: R.mergeDeepLeft(options, getDefaultOptions())
    });
  }, [options, data, type]);
  (0, _react.useEffect)(function () {
    if ((0, _transforms.nilOrEmpty)(points)) return; // console.log('Chart useEffect points:', points);

    R.map(pointIntoChart(chart.current, {
      length: length,
      ttl: ttl
    }))(points);
    chart.current.update();
  }, [points, length, ttl]);
  (0, _react.useEffect)(function () {
    if ((0, _transforms.nilOrEmpty)(point)) return; // console.log('Chart useEffect point:', point);

    pointIntoChart(chart.current, {
      length: length || 10,
      ttl: ttl
    }, point);
    chart.current.update();
  }, [point, length, ttl]);

  var canvasStyle = _objectSpread({}, defaultCanvasStyle, {}, style); // canvasStyle.minHeight = canvasStyle.height;


  return (0, _react.useMemo)(function () {
    return /*#__PURE__*/_react["default"].createElement("canvas", {
      // id={chartId}
      ref: canvasRef,
      width: canvasStyle.width,
      height: canvasStyle.height,
      style: canvasStyle
    });
  }, [canvasStyle]);
};

exports.Chart = Chart;
Chart.propTypes = propTypes;